module Brick(
  Brick,
  initBricks,
  isPositionInsideBrick
) where

import GameConstants
import Util

import qualified Graphics.UI.GLUT as GL

data Brick = Brick { position :: Position }

instance Renderable Brick where
  render b = GL.preservingMatrix $ do
    translateTo . position $ b
    GL.scale defaultBrickWidth defaultBrickHeight (gameDepth / 2)
    GL.renderObject GL.Wireframe (GL.Cube 1.0)

initBricks :: [Brick]
initBricks = map Brick pos's
  where
    dbw2 = defaultBrickWidth / 2
    dbh2 = defaultBrickHeight / 2
    pad = 10
    yOffset = (gameHeight / 2) + defaultBrickHeight
    xPos's = [(pad + dbw2), (pad + dbw2 + defaultBrickWidth) .. (gameWidth - (pad + dbw2))]
    yPos's = [(yOffset + dbh2), (yOffset + dbh2 + defaultBrickHeight) .. (gameHeight - (dbh2 + pad))]
    pos's = [Position x y | x <- xPos's, y <- yPos's]

defaultBrickWidth :: Double
defaultBrickWidth = 20

defaultBrickHeight :: Double
defaultBrickHeight = 10

isPositionInsideBrick :: Position -> Brick -> Bool
isPositionInsideBrick pos brick = (x pos) > leftEdge && (x pos) < rightEdge &&
                                  (y pos) < topEdge  && (y pos) > bottomEdge
  where
    brickPos = position brick 
    leftEdge    = x brickPos - (defaultBrickWidth / 2)
    rightEdge   = x brickPos + (defaultBrickWidth / 2)
    topEdge     = y brickPos + (defaultBrickHeight / 2)
    bottomEdge  = y brickPos - (defaultBrickHeight / 2)
