module Game(
  Game, 
  playerMoving, 
  playerStoppedMoving,
  tick,
  initGame,
  positionCamera,
  render
) where

import Ball
import Brick
import GameConstants
import Paddle
import Util

import Control.Monad
import Graphics.UI.GLUT

data Game = Game {
  ball :: Ball,
  bricks :: [Brick],
  paddle :: Paddle,
  livesLeft :: Int
}


initGame :: Game
initGame = Game {
  ball = initBall,
  bricks = initBricks,
  paddle = initPaddle,
  livesLeft = 3
}

playerMoving :: Direction -> Game -> Game
playerMoving d = updatePaddle (setPaddleMoving d)

playerStoppedMoving :: Direction -> Game -> Game
playerStoppedMoving d = updatePaddle (stopPaddleMoving d)

tick :: Game -> Game
tick game
  | livesLeft game > 0 = handleCollisions . moveTheBall . updatePaddle movePaddle $ game
  | otherwise = game
  where
    moveTheBall :: Game -> Game
    moveTheBall game = case (moveBall . ball $ game) of
      BallOk newBall        -> game { ball = newBall }
      BallHitBottom newBall -> game { ball = newBall,
                                      livesLeft = (pred $ livesLeft game) }

updateBall :: (Ball -> Ball) -> (Game -> Game)
updateBall f g = g { ball = f (ball g) }

updatePaddle :: (Paddle -> Paddle) -> (Game -> Game)
updatePaddle f g = g { paddle = f (paddle g) }

handleCollisions :: Game -> Game
handleCollisions =  checkBallBatCollision .
                    checkBallBrickCollision 
  where
    checkBallBatCollision g = updateBall (checkPaddleCollision (paddle g)) g

    checkBallBrickCollision g = g {
      ball = spedUpBall,
      bricks = remainingBricks results
    }
      where
        results = checkBrickCollisions (ball g) (bricks g) 
        spedUpBall = (iterate increaseSpeed . newBall $ results) !! (length $ killedBricks results)


positionCamera :: Game -> IO ()
positionCamera = lookAtBall . ball
  

instance Renderable Game where
  render g = do
    renderBorder
    render . ball $ g
    render . paddle $ g
    renderLives . livesLeft $ g
    forM_ (bricks g) render

renderLives :: Int -> IO ()
renderLives lives = forM_ [1..lives] renderLife
  where
    renderLife x = preservingMatrix $ do
      translate (Vector3 (-lifeSize) (gameHeight - (lifeSize * 1.5 * fromIntegral x)) 0)
      renderObject Wireframe (Teapot (lifeSize / 2))

lifeSize :: Double
lifeSize = 30

renderBorder :: IO ()
renderBorder = preservingMatrix $ do
  translate (Vector3 (gameWidth / 2) (gameHeight / 2) 0)
  scale gameWidth gameHeight gameDepth
  renderObject Wireframe (Cube 1.0)
