module Main where

import Control.Concurrent.MVar
import Control.Monad.Reader
import Graphics.UI.GLUT
import System.Exit

import Game
import Util(Direction(..))

main :: IO ()
main = do
  getArgsAndInitialize

  gamePtr <- newMVar initGame  

  initialDisplayMode $= [WithDepthBuffer, DoubleBuffered]
  createWindow "Haskell Breakout!"
  fullScreen

  cursor        $= None
  normalize     $= Enabled

  reshapeCallback       $= Just reshape
  displayCallback       $= (display gamePtr)
  keyboardMouseCallback $= Just (keyboardMouse gamePtr)
  addTimerCallback 1 (timerTick gamePtr)

  mainLoop


reshape :: Size -> IO ()
reshape s@(Size w h) = do 
  viewport $= (Position 0 0, s)
  matrixMode $= Projection
  loadIdentity
  perspective 60.0 ((fromIntegral w) / (fromIntegral h)) 10 1000.0
  matrixMode $= Modelview 0
  loadIdentity

  postRedisplay Nothing


keyboardMouse :: MVar Game -> Key -> KeyState -> Modifiers -> Position -> IO ()
keyboardMouse _ (Char '\ESC') _ _ _          = exitWith ExitSuccess
keyboardMouse m (SpecialKey KeyLeft)  ks _ _ = modifyMVar_ m $ return . keyboardLeftRightHelper ks DirLeft
keyboardMouse m (SpecialKey KeyRight) ks _ _ = modifyMVar_ m $ return . keyboardLeftRightHelper ks DirRight
keyboardMouse m (Char 'r') Down _ _          = modifyMVar_ m $ return . const initGame
keyboardMouse _ _ _ _ _                      = return ()

keyboardLeftRightHelper :: KeyState -> Direction -> Game -> Game
keyboardLeftRightHelper Down = playerMoving
keyboardLeftRightHelper Up   = playerStoppedMoving


timerTick :: MVar Game -> IO ()
timerTick mvg = do
  modifyMVar_ mvg $ return . tick
  addTimerCallback 15 (timerTick mvg)
  postRedisplay Nothing


display :: MVar Game -> IO ()
display mvg = do
  clear [ColorBuffer, DepthBuffer]
  loadIdentity

  withMVar mvg $ \g -> do
    positionCamera g
    render g

  swapBuffers
  finish
  get errors >>= mapM_ print
