module Paddle(
  Paddle,
  initPaddle,
  setPaddleMoving,
  stopPaddleMoving,
  movePaddle,
  calculateReturnVector
) where

import GameConstants
import Util

import qualified Graphics.UI.GLUT as GL

data Paddle = Paddle {
  position :: Position,
  size     :: Size,
  moving   :: Maybe Direction
}
  
paddleSpeed :: Speed
paddleSpeed = 1.5

paddleHeightDepth :: Size
paddleHeightDepth = 3.0

initPaddle :: Paddle 
initPaddle = Paddle {
  position = Position (gameWidth / 2) (15),
  size     = 25.0,
  moving   = Nothing
}

setPaddleMoving :: Direction -> Paddle -> Paddle
setPaddleMoving d p = p { moving = Just d }

stopPaddleMoving :: Direction -> Paddle -> Paddle
stopPaddleMoving d p
  | moving p == Just d = p { moving = Nothing }
  | otherwise          = p

movePaddle :: Paddle -> Paddle
movePaddle p = case (moving p) of
  Nothing         -> p
  (Just DirLeft)  -> checkWall p { position = pos { x = (xp - paddleSpeed) } }
  (Just DirRight) -> checkWall p { position = pos { x = (xp + paddleSpeed) } }
  where
    pos@(Position xp _) = position p

checkWall :: Paddle -> Paddle
checkWall p
  | xp - spo2 < 0         = p { position = pos { x = spo2 }}
  | xp + spo2 > gameWidth = p { position = pos { x = gameWidth - spo2 }}
  | otherwise  = p
  where
    spo2 = size p / 2
    pos@(Position xp _) = position p

calculateReturnVector :: Paddle -> Position -> Maybe Vector
calculateReturnVector paddle pos
  | (x pos) > leftEdge && (x pos) < rightEdge &&
    (y pos) < topEdge  && (y pos) > bottomEdge = Just . normalize $ (Vector (x pos - x paddlePos) (paddleHeightDepth))
  | otherwise = Nothing
  where
    paddlePos = position paddle
    leftEdge    = x paddlePos - ((size paddle) / 2)
    rightEdge   = x paddlePos + ((size paddle) / 2)
    topEdge     = y paddlePos + (paddleHeightDepth / 2)
    bottomEdge  = y paddlePos - (paddleHeightDepth / 2)

instance Renderable Paddle where
  
  render p = GL.preservingMatrix $ do
    translateTo (position p)
    GL.scale (size p) paddleHeightDepth paddleHeightDepth
    GL.renderObject GL.Wireframe (GL.Cube 1.0)
