module Util where

import {-# SOURCE #-} Ball
import {-# SOURCE #-} Brick

import qualified Graphics.UI.GLUT as GL

type Speed       = Double
type Size        = Double
type WhereOnEdge = Double

data Direction = DirLeft | DirRight
  deriving Eq

data Vector = Vector {
  i :: Double,
  j :: Double
}

data Position = Position {
  x :: Double,
  y :: Double
}

normalize :: Vector -> Vector
normalize (Vector x y) = Vector x' y'
  where
    x' = x / len
    y' = y / len
    len = sqrt (x*x + y*y)

class Renderable a where
  render :: a -> IO ()

translateTo :: Position -> IO ()
translateTo (Position x y) = GL.translate (GL.Vector3 x y 0)

(.+.) :: Position -> Vector -> Position
(Position x y) .+. (Vector x' y') = (Position (x + x') (y + y'))

(.*.) :: Double -> Vector -> Vector
n .*. (Vector x y) = Vector (x*n) (y*n)

data BallBrickCollisionResults = BBCR {
  newBall :: Ball,
  remainingBricks :: [Brick],
  killedBricks :: [Brick]
}

data BallMoveResult = BallHitBottom Ball | BallOk Ball
